Array.prototype.withUUID = function(uuid, fn) {
	var self = this;
	this.forEach(function(value, i) {
		if (value.uuid == uuid) {
			return fn(value, i, self);
		}
	});
};

((app) => {
	app.cache = {};

	app.load = async(filename) => {
		if (!(filename in app.cache)) {
			app.cache[filename] = await $.ajax(filename, {dataType: "text"}, null);
		}
		return app.cache[filename];
	};

	app.config(function($provide, $locationProvider, $routeProvider, $translateProvider, $mdThemingProvider, $mdAriaProvider) {
		$provide.factory("$routeProvider", () => $routeProvider);
		$provide.factory("$app", () => app);
		$mdThemingProvider.theme('error'); //register the themes. Because reasons
		$mdThemingProvider.theme('prompt');
		$mdThemingProvider.theme('warning');
		$mdAriaProvider.disableWarnings(); // disable aria-title warnings
		$locationProvider.html5Mode(true);
		$translateProvider.preferredLanguage('en');
		$translateProvider.fallbackLanguage('en');
		$translateProvider.useLoader('translation_loader');
		$translateProvider.useSanitizeValueStrategy('escapeParameters');
		$routeProvider.when("/:page", {
			templateUrl: function(params) {
				return `/templates/${params.page}.html?${Date.now()}`;
			},
			controller: function(fn, $scope, $rootScope, $routeParams, $controller) {
				$rootScope.current_view = $routeParams.page;
				$controller(fn, { $scope: $scope });
			},
			resolve: {
				fn: async ($route) => {
					const name = $route.current.params.page;
					// $routeProvider doesn't handle errors, do it ourselves
					try {
						return Function(await app.load(`controllers/${name}.js?${Date.now()}`))();
					} catch (error) {
						console.error(error);
					}
				},
			},
		});
		$routeProvider.otherwise({redirectTo: "/tenants"});
	});
	app.controller('navbar', function($scope, $mdPanel, $translate, ws) {
		$scope.toggle_collapse = function() {
			$scope.collapse = ($scope.collapse) ? false : true;
		};

		angular.element(document).bind('click', function(event) { // close user profile menu on click
			if($scope.panel != undefined) { $scope.panel.close(); }
		})

		$scope.profile_menu = function($event) {

			function PanelMenuCtrl(mdPanelRef, $scope, ws) {
				this.go = function(destination) {
					app.jump(destination);
				};

				this.closeMenu = function() {
					mdPanelRef && mdPanelRef.close();
				};
				ws.profile().then(function(json) {
					$scope.profile = json.data;
				});

				this.logout = function() {
					$scope.visible = false; // close about dialog if open
					ws.logout(sessionStorage.session_id);
					app.jump('/login');
				}
				this.about = app.show_about;
			}

			var position = $mdPanel.newPanelPosition()
				.relativeTo($event.target.closest('header'))
				.addPanelPosition(
					$mdPanel.xPosition.ALIGN_END,
					$mdPanel.yPosition.BELOW
				).withOffsetX(-18);
			var config = {
				attachTo: angular.element(document.body),
				controller: PanelMenuCtrl,
				locals: {
					items: $scope.profile
				},
				controllerAs: 'ctrl',
				templateUrl: `/templates/navbar.html?${Date.now()}`,
				position: position,
				panelClass: 'menu-panel-container',
				openFrom: $event,
				focusOnOpen: false,
				clickOutsideToClose: true,
				propagateContainerEvents: true,
				groupName: 'menus'
			};
			$mdPanel.open(config).then(function(panel) {
				$scope.panel = panel;
			});
		}

		$scope.toggle_notifications = function() {
			app.state = !$scope.state;
		};

		var update_pending = function() {
			$scope.has_pending = false;
			angular.forEach($scope.notifications, function(item) {
				if(item.state.indexOf('pending') >= 0) {
					$scope.has_pending = true;
					return true;
				}
			})
		}

		var append = function(message) {
			if(message.event == 'error') {
				message.data.name = $translate.instant('global.error');
				message.data.request = message.data.reason;
			}
			message.data.state = message.event;
			message.data.date = new Date();
			$scope.notifications.push(message.data);
			update_pending();
		};

		var remove = function(message) {
			$scope.notifications.forEach(function(value, i) {
				if (value.task_id == message.data.task_id) {
					$scope.notifications.splice(i, 1);
				}
			});
			update_pending();
		};

		var on_error = function(message) {
			$scope.notifications.forEach(function(value, i) {
				if (value.task_id == message.data.task_id) {
					$scope.notifications[i].reason = message.data.reason;
					$scope.notifications[i].state = message.event;
				}
			});
			update_pending();
		};

		var update = function(message) {
			$scope.notifications.forEach(function(value, i) {
				if (value.task_id == message.data.task_id) {
					$scope.notifications[i].state = message.event;
				}
			});
			update_pending();
		}

		$scope.notifications = [];
		$scope.has_pending = $scope.show_cancel_button = false;

		ws.on('pending-start-container', append, $scope);
		ws.on('pending-stop-container', append, $scope);
		ws.on('container-upgrade-pending', append, $scope);

		ws.on('updating-container', append, $scope);
		ws.on('container-updated', remove, $scope);

		ws.on('creating-container', append, $scope);
		ws.on('container-created', remove, $scope);

		ws.on('removing-container', append, $scope);
		ws.on('container-removed', remove, $scope);

		ws.on('starting-container', update, $scope);
		ws.on('container-started', remove, $scope);

		ws.on('stopping-container', update, $scope);
		ws.on('container-stopped', remove, $scope);

		ws.on('upgrading-container', update, $scope);
		ws.on('container-upgraded', remove, $scope);

		ws.on('error', on_error, $scope);

		$scope.reverse = function(array) {
			var copy = [].concat(array);
			return copy.reverse();
		};
		$scope.close = function(id) {
			index = $scope.notifications.length - id - 1;
			$scope.notifications.splice(index, 1);
		}
	});

	app.controller('footer', function($scope, $translate) {
		update_visibility = function() {
			$scope.visible = $scope.current_view != 'login' && ($scope.remove || $scope.cancel || $scope.submit || false);
		};

		app.listing = function() {
			$scope.remove = false;
			$scope.cancel = false;
			$scope.submit = false;
			$scope.reset = false;
			update_visibility();
		};

		app.creating = function(s, confirm_label, cancel_label) {
			if(confirm_label == null) {
				confirm_label = 'footer.create'
			}
			if(cancel_label == null) {
				cancel_label = 'footer.back'
			}
			$scope.remove = false;
			$scope.cancel = s.cancel;
			$scope.submit = s.submit;
			$scope.reset = s.reset;
			$scope.cancel_label = $translate.instant(cancel_label);
			$scope.confirm_label = $translate.instant(confirm_label);
			update_visibility();
		};

		app.editing = function(s) {
			$scope.remove = s.remove;
			$scope.cancel = s.cancel;
			$scope.submit = s.submit;
			$scope.confirm_label = $translate.instant('footer.save');
			$scope.cancel_label = $translate.instant('footer.back');
			update_visibility();
		};
	});

	app.controller('aside', function($scope) {
		$scope.go = function(destination) {
			app.jump(destination);
		}
	});

	app.controller('about', function($scope, ws, $translate, about_dialog) {
		app.show_about = function() {
			ws.version().then(function(json) {
				$scope.text = $translate.instant(
					"about.project_version", {
						project: "Xorcom MT Manager",
						version: json.data
					}
				);
				about_dialog.show();
			});
		};
		$scope.hide_about = function() {
			about_dialog.hide();
		}
	});

	app.directive('tabs', function() {
		return {
			scope: false,
			restrict: 'A',
			controller: ['$scope', function tabsController($scope) {
				$scope.tabs = [];

				this.addTab = function(tab) {
					$scope.tabs.push(tab);
				};

				this.isTabActive = function(link) {
					for (var i = 0; i < $scope.tabs.length; i++) {
						if ($scope.tabs[i].link === link) {
							return $scope.tabs[i].active;
						}
					}
					return false;
				};

				this.setTabToActive = function(link) {
					for (var i = 0; i < $scope.tabs.length; i++) {
						$scope.tabs[i].active = $scope.tabs[i].link === link;
					}
				};
			}]
		}
	});

	app.directive('tabsLink', function() {
		return {
			scope: false, // separate scope from outside and inside directive
			restrict: 'A', // atributes (for data-tabs-link)
			require: '^tabs', // hook for controller on own or parent
			link: function(scope, element, attributes, controller) {
				var tab = {
					link: attributes.tabsLink,
					active: false
				};
				controller.addTab(tab);

				element.bind('click', function() {
					scope.$apply(function() {
						if(tab.active != true) {
							controller.setTabToActive(tab.link);
						}
					});
				});

				scope.$watch('tabs', function() {
					element.toggleClass('active', controller.isTabActive(tab.link));
				}, true);
			}
		};
	});

	app.directive('tabsPane', function() {
		return {
			scope: false,
			restrict: 'A',
			require: '^tabs',
			link: function(scope, element, attributes, controller) {
				if (element.hasClass('active')){
					controller.setTabToActive(attributes.tabsPane);
				}

				scope.$watch('tabs', function() {
					element.toggleClass('active', controller.isTabActive(attributes.tabsPane));
				}, true);
			}
		};
	});

	app.directive("whitelistTemplate", function() {
		return {
			restrict : "E", // only match element name
			replace : false,
			templateUrl: `/templates/whitelist.html?${Date.now()}`,
			$scope: {
				tenant: "=",
			},
			controller: ['$scope', function Controller($scope) {
				$scope.add_row = function() {
					if($scope.whitelist == undefined) {
						$scope.whitelist = []
					}
					var len = ($scope.services == undefined) ? 0 : $scope.services.length;
					$scope.whitelist.push({
						uuid:null,
						description:null,
						address:null,
						enabled: (new Array(len)).fill(false)
					});
				};

				$scope.remove_row = function(index) {
					if ($scope.whitelist != undefined && typeof $scope.whitelist[index] != 'undefined') {
						$scope.whitelist.splice(index, 1);
					}
				};
			}]
		};
	});

	app.controller('dimmer', function($scope, about_dialog) {
		$scope.hide_about = function() {
			about_dialog.hide();
		}
	});

	app.directive('ngEnter', function () {
		return function (scope, element, attrs) {
			element.bind("keydown keypress", function (event) {
				if (event.which === 13) {
					scope.search = function() {
						app.jump('/search?q='+scope.search_string);
					}
					scope.$apply(function () {
						scope.$eval(attrs.ngEnter);
					});
					event.preventDefault();
				}
			});
		};
	});

	// parse injected HTML
	app.directive('compile', ['$compile', function ($compile) {
		return function(scope, element, attrs) {
			scope.$watch(
				function(scope) {
					return scope.$eval(attrs.compile);
				},
				function(value) {
					element.html(value);
					$compile(element.contents())(scope);
				}
			);
		};
	}])

	app.run(function(ws, $rootScope, $location, $mdDialog, $mdToast, $translate, toaster, message) {
		$rootScope.app = app;
		app.jump = function(destination) {
			app.referrer = location.pathname + location.search;
			$location.url(destination);
		}

		app.tabs = function($route) {
			if($route.current.params.tab) { // set active tab
				var active_tab = $route.current.params.tab;
				angular.forEach(angular.element(document.querySelectorAll('tab')), function(tab) {
					tab.classList.remove('active');
				});
				angular.element(document.querySelector('tab[data-tabs-pane="'+active_tab+'"]')).addClass('active');
				angular.element(document.querySelector('a[data-tabs-link="'+active_tab+'"]')).addClass('active');
			}
		}

		app.default_error_handler = function(error) {
			console.error(error);
			message = error.message;
			if(error.status == 'rejected' && error.message == null){
				message = $translate.instant('global.action_denied');
			}
			$mdDialog.show(
				$mdDialog.alert()
					.theme('error')
					.parent(angular.element(document.querySelector('#popupContainer')))
					.clickOutsideToClose(true)
					.title($translate.instant('global.error'))
					.htmlContent(message)
					.clickOutsideToClose(true)
					.ok($translate.instant('global.ok'))
			);
		};
		app.jump('/login');
		ws.on('connect', function () {
			$rootScope.connected = true;
			if (app.config_test_dialog) {
				$mdDialog.hide();
				delete app.config_test_dialog;
			}
			if (sessionStorage.session_id) {
				ws.authenticate(sessionStorage.session_id).then(function(json) {
					$rootScope.is_system_owner = json.data.is_system_owner;
					app.jump(app.referrer);
					if (json.data.notify_whitelist_unconfigured) {
						message.prompt_2();
					}
				}).catch(function() {
					app.jump('/login');
				});
			}
			if (app.confirm_token) {
				ws.confirm_changes(app.confirm_token).then(function() {
					toaster.success($translate.instant('global.saved'));
				}).catch(app.default_error_handler);
				delete app.confirm_token;
			}
		});
		ws.on('disconnect', function() {
			if (app.confirm_token == undefined) {
				$rootScope.connected = false;
				app.state = false;
			}
		});

		$rootScope.$on('$translateChangeSuccess', function() {
			$rootScope.localization_loaded = true;
		});

		var bump_numevents = function() {
			if(location.pathname != '/event-log') {
				$rootScope.numevents = ($rootScope.numevents || 0) + 1;
			}
		};

		ws.on('container-created', bump_numevents, $rootScope);
		ws.on('container-started', bump_numevents, $rootScope);
		ws.on('container-stopped', bump_numevents, $rootScope);
		ws.on('container-removed', bump_numevents, $rootScope);
		ws.on('container-renamed', bump_numevents, $rootScope);
		ws.on('container-system-name-changed', bump_numevents, $rootScope);
		ws.on('container-status-changed', bump_numevents, $rootScope);
		ws.on('container-extensions-changed', bump_numevents, $rootScope);
		ws.on('container-version-changed', bump_numevents, $rootScope);
		ws.on('container-serial-changed', bump_numevents, $rootScope);
		ws.on('error', bump_numevents);
	});

	app.filter('seconds_to_date', [function() {
		return function(seconds) {
			return new Date(1970, 0, 1).setSeconds(seconds);
		};
	}])

	app.directive('popup', function() {
		return {
			link: function (scope, element, attrs) {
				scope.$on('eventFired', function(event, message) {
					scope.errors = [ message.data ];
					scope.show = true;
				});
			}
		}
	});
})(angular.module('app', ['cpbx', 'ngRoute', 'pascalprecht.translate', 'isteven-multi-select', 'ngMaterial', 'ngMessages', 'toastr', 'ngSanitize', 'ngAnimate', 'angularUtils.directives.dirPagination', 'ui.bootstrap']));
