return function ($app, ws, $q, $scope, $location, $translate, $mdPanel, toaster, $rootScope, multiselect, message, $route) {
	$scope.submit = function() {
		$scope.filter();
		if($scope.valid()) {
			if ($rootScope.is_system_owner && $scope.tenant != undefined) {
				$scope.tenant.owner = "";
				angular.forEach($scope.assigned_reseller, function(reseller) {
					$scope.tenant.owner = reseller.uuid;
				});
			}
			ws.create_container($scope.tenant, $scope.whitelist, $scope.services, $scope.apps).then(function(json) {
				$app.jump('/tenants');
				toaster.success($translate.instant('global.pending'));
			}).catch($app.default_error_handler);
		}
	};
	$scope.isActive = true;
	$scope.cancel = function() {
		$app.jump('/tenants');
	};

	var add_to_scope = function(name, internal_port) {
		if($scope.services != undefined) {
			$scope.services.push({ // add to whitelist columns
				'name': name,
				'internal_port': internal_port,
				'forwarding': true,
			});
			angular.forEach($scope.whitelist, function(whitelist) {
				whitelist.enabled.push(false); //add column to existing rows
			});
		}
	}

	$scope.add_service = function() {
		if(valid_service()) {
			add_to_scope($scope.new_service.name, $scope.new_service.internal_port);
			$scope.new_service = {}; // clean the new row
		}
	};

	var valid_service = function() {
		var errors = [];
		if(typeof $scope.new_service == "undefined" ||
			!('name' in $scope.new_service) ||
			!('internal_port' in $scope.new_service) ||
			$scope.new_service.name.length == 0 ||
			$scope.new_service.internal_port.length == 0
		) {
			errors['undefined'] = $translate.instant('tenant.service.undefined');
		}
		if(typeof $scope.new_service != "undefined" &&
			'internal_port' in $scope.new_service &&
			$scope.new_service.internal_port.length > 0 &&
			!isNumeric($scope.new_service.internal_port)
		){
			errors['internal_port'] = $translate.instant('tenant.service.internal_port.invalid');
		}
		if(Object.keys(errors).length > 0) {
			message.error(errors);
			return false;
		}
		return true;
	}

	var isNumeric = function(n) {
		return !isNaN(parseFloat(n)) && isFinite(n);
	}

	$scope.remove_service = function(index) {
		message.confirm({'content' : $translate.instant('tenant.service.remove.warning')}).then(function(result) {
			if(result == true) {
				if ($scope.services != undefined && typeof $scope.services[index] != 'undefined') {
					$scope.services.splice(index, 1);
				}
				angular.forEach($scope.whitelist, function(whitelist) {
					whitelist.enabled.splice(index, 1); //add column to existing rows
				});
			}
		}, function() {});
	};

	// needed to be grayed out for now. So hack it
	$scope.sip_index = null;
	angular.forEach($scope.services, function(service, index) {
		if(service.name == 'SIP') {
			$scope.sip_index = index;
		}
	});

	$scope.change_limit = function() {
		$scope.isActive = true;
		if($scope.tenant.resource_plan == "") {
			$scope.isActive = false;
		}
		$scope.resource_limits.withUUID($scope.tenant.resource_plan, function(limit) {
			$scope.tenant.memory = limit.memory;
			$scope.tenant.cpu = limit.cpu;
			$scope.tenant.diskspace = limit.diskspace;
		});
	}

	$scope.resellers = [];
	$scope.whitelist = [];
	$scope.multiselect_l18n = multiselect.localization();
	$app.creating($scope);

	$scope.dismissDialog = function($event) {
		if($scope.popover != undefined) {
			$scope.popover.close();
		}
	}
	$scope.showDialog = function ($event, label) {
		var position = $mdPanel.newPanelPosition()
		.relativeTo($event.target)
		.addPanelPosition(
			$mdPanel.xPosition.ALIGN_START,
			$mdPanel.yPosition.BELOW
		);
		var config = {
			attachTo: angular.element(document.body),
			template:
			`<div class="popover" role="tooltip">
				<h3 class="popover-header"> {{ 'tenant.` + label + `' | translate }}</h3>
				<div class="popover-body">
					{{'tenant.` + label + `.tooltip' | translate}}
				</div>
			</div>`,
			position: position,
			panelClass: 'popover_panel',
			openFrom: $event,
			focusOnOpen: false,
			clickOutsideToClose: true,
			escapeToClose: true,
			propagateContainerEvents: true,
		};
		$mdPanel.open(config).then(function(panel) {
			$scope.popover = panel;
		});
	}

	$scope.app_prompt = function(index) {
		const app = $scope.apps[index];
		message.prompt_1({
			'title': app.displayname,
			'warning' : $translate.instant('tenant.active_paid_feature_warning'),
			'content' : app,
			'popup_text' : $translate.instant(`app.${app.name}.popup`),
		}).then(function() {
			app.status = false;
			if (app.toggle_status) {
				app.status = true;
				message.confirm_1({
					'title': 'global.note',
					'confirm': 'tenant.confirm',
					'cancel': 'tenant.cancel',
					'content' : 'tenant.full_upgrade_required',
				}).then(function(result) {
					if (!result) {
						app.status = false;
					}
				});
			}
		});
	};

	$scope.show_info = function(index) {
		message.info({'title': 'activate' , 'content' : $scope.apps[index]});
	};

	//filter in reverese to avoid array mutation
	$scope.filter = function() {
		if($scope.whitelist != undefined && $scope.whitelist.length > 0) {
			for(var i = $scope.whitelist.length - 1; i >= 0 ; i--) {
				if ($scope.whitelist[i].address == undefined || $scope.whitelist[i].address.length == 0) {
					$scope.whitelist.splice(i, 1);
				}
			}
		}
	}

	$scope.valid = function() {
		var errors = [];
		if($scope.tenant.name == undefined || $scope.tenant.name.length == 0) {
			errors['name'] = $translate.instant('tenant.name.invalid');
		}
		if($scope.tenant.image == undefined) {
			errors['image'] = $translate.instant('tenant.image.invalid');
		}
		if($scope.tenant.limit == undefined ) {
			errors['resource_plan'] = $translate.instant('tenant.resource_plan.invalid');
		} else if($scope.tenant.limit == 0) { // custom
			if($scope.tenant.diskspace == undefined ||
				$scope.tenant.diskspace.length == 0 ||
				parseInt($scope.tenant.diskspace) == 0 ||
				parseInt($scope.tenant.diskspace) < 4
			) {
				errors['diskspace'] = $translate.instant('resource.diskspace.invalid');
			}
			if($scope.tenant.memory == undefined || $scope.tenant.memory < 0.5) {
				errors['memory'] = $translate.instant('resource.memory.invalid');
			}
			if($scope.tenant.cpu == undefined || isNaN($scope.tenant.cpu) || $scope.tenant.cpu.length == 0 || $scope.tenant.cpu < 0.1) {
				errors['cpu'] = $translate.instant('resource.cpu.invalid');
			} else {
				$scope.tenant.cpu = Math.round(parseFloat($scope.tenant.cpu) * 10) / 10;
				if($scope.tenant.cpu < 0.1) {
					errors['cpu'] = $translate.instant('resource.cpu.invalid');
				}
			}
		}
		if($scope.whitelist.length != 0) {
			if($scope.settings.interface == undefined || $scope.settings.interface.length == 0) {
				errors['interface'] = $translate.instant('settings.interface.invalid');
			}
			var used_addresses = [];
			angular.forEach($scope.whitelist, function(whitelist) {
				if(whitelist.address != undefined && whitelist.address.length > 0) {
					if(used_addresses.indexOf(whitelist.address) !== -1) {
						errors['duplicate_address'] = $translate.instant('settings.whitelist.ip.duplicate');
					}
					used_addresses.push(whitelist.address);
					if(whitelist.ami == false && whitelist.ssh == false &&
						whitelist.sip == false && whitelist.web == false
					) {
						errors['whitelist'] = $translate.instant('settings.whitelist.service.invalid');
					}
				}
			});
		}
		const basic_max = $scope.apps.find(app => app.name == "tmx-basic")?.max ?? -1;
		const advanced_max = $scope.apps.find(app => app.name == "tmx-advanced")?.max ?? -1;
		if (basic_max == 0 && advanced_max == 0) {
			errors['max'] = $translate.instant('tenant.zero_extensions');
		}
		if(Object.keys(errors).length > 0) {
			message.error(errors);
			return false;
		}
		return true;
	}
	$app.tabs($route);

	$q.all([
		ws.apps(),
		ws.whitelist_services(),
		ws.resource_limits(),
		ws.images(),
		ws.settings(),
		ws.profile(),
		ws.resellers(),
	]).then(([apps, services, resource_plans, images, settings, profile, resellers]) => {
		$scope.apps = [];
		for (const name in apps.data) {
			const url = $translate.instant(`app.${name}.url`);
			$scope.apps.push({
				name: name,
				description: $translate.instant(`app.${name}.description`),
				displayname: $translate.instant(`app.${name}.displayname`),
				unit: $translate.instant(`app.${name}.unit`),
				status: false,
				usage: "-",
				icon: $translate.instant(`app.${name}.icon`),
				url: (url != `app.${name}.url`) ? url : null,
				no_max: apps.data[name].no_max,
				max: apps.data[name].max_default >= 0 ? apps.data[name].max_default : null,
				force_enable: apps.data[name].force_enable,
				info: $translate.instant(`app.${name}.details`) != "",
			});
		}
		// extension apps are in fixed order, other apps are sorted by name
		$scope.apps = [
			$scope.apps.find(app => app.name == "extensions"),
			$scope.apps.find(app => app.name == "tmx-basic"),
			$scope.apps.find(app => app.name == "tmx-advanced"),
		].filter(app => !!app).concat(
			$scope.apps
				.filter(app => !["extensions", "tmx-basic", "tmx-advanced"].includes(app.name))
				.sort((a, b) => a.displayname.localeCompare(b.displayname))
		);
		$scope.services = services.data;
		angular.forEach(resource_plans.data, function(limit) {
			limit.memory = Math.round(limit.memory * 100) / 100;
		});
		$scope.resource_limits = resource_plans.data;
		if ($rootScope.is_system_owner) {
			$scope.resource_limits.push({
				uuid:"",name: $translate.instant('tenant.custom'), description:$translate.instant('tenant.custom'),
				memory: 0.5, cpu: 0.8, diskspace: 4
			});
		}
		$scope.change_limit();
		$scope.images = Object.entries(images.data).map(([alias, image]) => {
			let name = alias;
			if (image.version != "") {
				name = `${name} (${image.version})`;
			}
			if (image.description != "") {
				name = `${name} - ${image.description}`;
			}
			return { name, alias };
		});
		$scope.settings = settings.data.settings;
		if ($rootScope.is_system_owner && !$scope.settings.username) {
			$scope.style = { display: 'block' };
		}
		$scope.tenant.image = null;
		if ($scope.settings.pbx_image) {
			for (const image of $scope.images) {
				if (image.alias == $scope.settings.pbx_image) {
					$scope.tenant.image = image;
					break;
				}
			}
		}
		$scope.profile = profile.data;
		$scope.resellers = resellers.data;
	}).catch($app.default_error_handler);
};
