return function ($app, ws, $q, $scope, $translate, toaster, $uibModal, $route) {
	$scope.itemsPerPage = 25;
	$scope.currentPage = 1;

	$q.all([
		ws.settings(),
		ws.containers(),
		ws.did_routing(),
	]).then(([settings, containers, routes]) => {
		$scope.settings = settings.data.settings;
		// FIXME: why is this a string?
		$scope.did_routes_enabled = settings.data.settings.did_routes_enabled == "1";
		$scope.tenants = containers.data;
		$scope.tenants.sort(function(a, b){
			if(a.name < b.name) { return -1; }
			if(a.name > b.name) { return 1; }
			return 0;
		});
		$scope.did_routing = routes.data;
	}).catch($app.default_error_handler);

	$scope.remove_did_route_row = function(index) {
		$uibModal.open({
			templateUrl: `/templates/remove_global_did_route_modal.html?${Date.now()}`,
			backdrop: 'static',
			backdropClass: 'modal-backdrop',
			animation: false,
			windowClass: 'show',
			scope: $scope,
			controller: function($scope, $uibModalInstance) {
				$scope.cancel = function() {
					$uibModalInstance.close();
				};
				$scope.ok = function() {
					var true_index = (($scope.currentPage - 1) * $scope.itemsPerPage) + index; // index start from 0
					if ($scope.did_routing && $scope.did_routing[true_index]) {
						$scope.did_routing.splice(true_index, 1);
					}
					toaster.success_centered($translate.instant('did_routing.route_removed'));
					$uibModalInstance.close();
				};
			},
		});
	}

	$scope.edit_did_route = function(data) {
		$uibModal.open({
			templateUrl: `/templates/edit_global_did_route_modal.html?${Date.now()}`,
			backdrop: 'static',
			backdropClass: 'modal-backdrop',
			animation: false,
			windowClass: 'show',
			controller: function($scope, $uibModalInstance, ws) {
				var vm = this;
				$scope.uuid = data.uuid;
				$scope.pattern = data.pattern;

				vm.editedItem = angular.copy(data);
				$scope.cancel = function() {
					$uibModalInstance.close();
				};
				$scope.ok = function() {
					if($scope.uuid == null) {
						toaster.error($translate.instant('did_routing.uuid_empty'));
					} else if($scope.pattern == null) {
						toaster.error($translate.instant('did_routing.pattern_empty'));
					} else {
						vm.editedItem.uuid = $scope.uuid;
						vm.editedItem.pattern = $scope.pattern;
						$scope.tenants.withUUID($scope.uuid, function(tenant) {
							vm.editedItem.name = tenant.name;
						});
						angular.copy(vm.editedItem, data);
						$uibModalInstance.close();
						toaster.success_centered($translate.instant('did_routing.updated'));
					}
				}
				ws.containers().then(function(json) {
					$scope.tenants = json.data;
					$scope.tenants.sort(function(a, b){
						if(a.name < b.name) { return -1; }
						if(a.name > b.name) { return 1; }
						return 0;
					})
				})
			},
		});
	};

	$scope.add_did_route_row = function(did_routing) {
		$uibModal.open({
			templateUrl: `/templates/add_global_did_route_modal.html?${Date.now()}`,
			backdrop: 'static',
			backdropClass: 'modal-backdrop',
			animation: false,
			windowClass: 'show',
			controller: function($scope, $uibModalInstance, ws) {
				$scope.cancel = function() {
					$uibModalInstance.close();
				};
				$scope.ok = function() {
					if($scope.uuid == null) {
						toaster.error($translate.instant('did_routing.uuid_empty'));
					} else if($scope.pattern == null) {
						toaster.error($translate.instant('did_routing.pattern_empty'));
					} else if(!$scope.pattern_valid($scope.pattern)) {
						toaster.error($translate.instant('did_routing.pattern_invalid'));
					} else {
						$scope.tenants.withUUID($scope.uuid, function(tenant) {
							$scope.name = tenant.name;
						});
						did_routing.push({
							'uuid': $scope.uuid,
							'pattern': $scope.pattern,
							'name': $scope.name,
						});
						$uibModalInstance.close();
						toaster.success_centered($translate.instant('did_routing.new_route_added'));
					}
				}
				ws.containers().then(function(json) {
					$scope.tenants = json.data;
					$scope.tenants.sort(function(a, b){
						if(a.name < b.name) { return -1; }
						if(a.name > b.name) { return 1; }
						return 0;
					})
				})

				$scope.pattern_valid = function(pattern) {
					if(pattern.length > 0) {
						var res = pattern.match(/^[0-9a-zA-Z?*,.+-_^\]\[]+$/g);
						if(res != null) {
							return true;
						}
					}
					return false;
				}
			},
		});

	};

	$scope.import_did_routes = function() {
		var filename = document.getElementById("import_did");
		if(filename.files[0] == undefined) {
			toaster.warning($translate.instant('did_routing.no_file_selected'));
		} else {
			$uibModal.open({
				templateUrl: `/templates/import_did_routes_modal.html?${Date.now()}`,
				backdrop: 'static',
				backdropClass: 'modal-backdrop',
				animation: false,
				windowClass: 'show',
				scope: $scope,
				controller: function($uibModalInstance, ws) {
					$scope.cancel = function() {
						$uibModalInstance.close();
					};
					$scope.ok = function() {
						$uibModalInstance.close();
						var filename = document.getElementById("import_did");
						var reader = new FileReader();
						reader.onload = function(e) {
							if (reader.result && reader.result.length > 0) {
								ws.import_didrouting(reader.result).then(function(result) {
									if(result.data.status.toLowerCase() == 'success') {
										toaster.success_centered($translate.instant(result.data.status));
									} else {
										toaster.error($translate.instant('did_routing.failed_no_changes_applied'));
									}
									$scope.did_routing = result.data.didroutes;
									$scope.log = result.data.log;
								}).catch((error) => {
									console.error(error);
								});
							}
						};
						reader.readAsDataURL(filename.files[0]);
					}
				},
			});
		}
	}

	$scope.submit = function() {
		$scope.filter_dids();
		ws.update_didrouting($scope.did_routing, $scope.did_routes_enabled).then(function() {
			$app.jump('/did_routing');
			toaster.success($translate.instant('global.saved'));
		}).catch($app.default_error_handler);
	}

	$scope.filter_dids = function() {
		angular.forEach($scope.did_routing, function(route, index) {
			if(route.pattern.length == 0) {
				$scope.did_routing.splice(index, 1);
			}
		});
	}

	$scope.export_didrouting = function(type) {
		ws.export_didrouting(type).then(function(json) {
			if(type == 'json') {
				download_file(JSON.stringify(json.data), 'did_routing.json');
			} else if (type == 'csv') {
				download_file(json.data, 'did_routing.csv');
			}
		});
	}

	var download_file = function(content, filename) {
		var a = document.createElement("a");
		document.body.appendChild(a); // needed for Firefox
		a.style = "display: none";
		var csv = new Blob([content], { type: 'text/csv' });
		a.href = URL.createObjectURL(csv);
		a.download = filename;
		a.click();
	};

	$scope.hide_footer = function() {
		$app.listing();
	};
	$scope.show_footer = function() {
		$app.creating($scope, 'footer.save');
	};
	$scope.show_footer();
	$app.tabs($route);
};
