return function ($app, ws, $q, $scope, $rootScope, $location, $translate, toaster, message, multiselect, $timeout, $interval, timer_clock, $route) {
	$app.dump_counter = 0;
	$scope.add_service_row = function() {
		$scope.services.push({ // add to whitelist columns
			'name': null,
			'internal_port': null,
			'global': true,
			'forwarding': true,
		});
	};
	$scope.remove_service_row = function(index) {
		if ($scope.services && $scope.services[index]) {
			$scope.services.splice(index, 1);
		}
	}

	var get_resource_limits = function(json) {
		ws.resource_limits().then(function(json) {
			angular.forEach(json.data, function(limit) {
				limit.memory = Math.round(limit.memory * 100) / 100;
			});
			$scope.resource_limits = json.data;
		});
	};

	$scope.toogle_dump = function() {
		if ($app.active_dump) { // stop dump
			$app.active_dump = false;
			ws.stop_dump($scope.dump_uuid).then(function(json) {
				timer_clock.stop();
				var data = atob(json.data.data);
				var bytes = new Uint8Array(data.length);
				for (var i = 0; i < data.length; ++i) {
					bytes[i] = data.charCodeAt(i);
				}
				var blob = new Blob([bytes], { type: 'application/vnd.tcpdump.pcap' });
				var a = document.createElement("a");
				document.body.appendChild(a); // needed for Firefox
				a.style = "display: none";
				a.href = URL.createObjectURL(blob);
				a.download = json.data.filename+'.pcap';
				a.click();
				$scope.dump_action = $translate.instant('settings.start');
				$timeout.cancel(timer); // stop "stop and download" action
			}).catch($app.default_error_handler);
		} else { // start dump
			$app.dump_counter = 10 * 60;
			$app.active_dump = true;
			ws.start_dump().then(function(json) {
				$scope.dump_uuid = json.data.uuid;
				$scope.dump_action = $translate.instant('settings.stop_n_download');
				$app.dump_uuid = $scope.dump_uuid; // keep dump state if nav. away

				timer_clock.start(); // start clock
				update_clock();
				timer = $timeout(function() { // stop and download timeout calls $scope.apply(). magic
					$scope.toogle_dump();
				}, 600000); // 10 minutes timeout.
			}).catch($app.default_error_handler);
		}
	}

	// update clock function
	var update_clock = function() {
		var ref;
		var tick = function() {
			$app.dump_counter = timer_clock.get();
			if ($app.dump_counter == 0) {
				if(ref) {
					$interval.cancel(ref);
				}
			}
		}
		ref = $interval(tick, 1000);
		return ref;
	}

	// Check if have active clock on comming back
	// if yes, restart clock update
	var ref;
	if($app.active_dump) {
		var ref = update_clock();
	} else {
		$interval.cancel(ref);
	}

	$scope.multiselect_l18n = multiselect.localization();
	$scope.submit = function() {
		if($scope.valid()) {
			$scope.settings.interface = $scope.interface = null;
			angular.forEach($scope.interfaces, function(interface) {
				$scope.settings.interface = interface.name;
			});
			$scope.settings.pbx_image = ($scope.output_image[0]) ? $scope.output_image[0].alias : null;
			$q.all([
				ws.update_resource_limits($scope.resource_limits),
				ws.update_whitelist_services($scope.services),
				ws.update_settings($scope.settings, $scope.whitelist),
			]).then(() => {
				$app.jump('/settings');
				get_resource_limits();
				toaster.success($translate.instant('global.saved'));
			}).catch($app.default_error_handler);
		}
	};

	$scope.valid = function() {
		var errors = [];
		angular.forEach($scope.resource_limits, function(limit) {
			if(limit.name == undefined ||  limit.name.length == 0 || limit.name.length > 10) {
				errors['name'] = $translate.instant('resource.plan_name.empty');
			}
			if(limit.diskspace == undefined ||
				limit.diskspace.length == 0 ||
				parseInt(limit.diskspace) == 0 ||
				parseInt(limit.diskspace) < 4
			) {
				errors['diskspace'] = $translate.instant('resource.diskspace.invalid');
			}
			if(limit.memory == undefined || limit.memory < 0.5) {
				errors['memory'] = $translate.instant('resource.memory.invalid');
			}
			if(limit.cpu == undefined || isNaN(limit.cpu) || limit.cpu.length == 0 || limit.cpu < 0.1) {
				errors['cpu'] = $translate.instant('resource.cpu.invalid');
			} else {
				limit.cpu = Math.round(parseFloat(limit.cpu) * 10) / 10;
				if(limit.cpu < 0.1) {
					errors['cpu'] = $translate.instant('resource.cpu.invalid');
				}
			}
		});

		var resources = [ 'cpu', 'memory' ] ;
		for(var i = 0; i <= resources.length - 1; i++) {
			var op = resources[i] + '_overprovision';
			if(isNaN($scope.settings[op])) {
				errors['op_value_' + resources[i]] = $translate.instant(
					'settings.over_provision.invalid',
					{ resource: $translate.instant('settings.' + resources[i]) });
			} else if($scope.settings[op].length == 0) {
				$scope.settings[op] = "100";
			}

			if($scope.allocated[resources[i]] > $scope.system_resources[resources[i] + '_total'].toFixed(1)) {
				errors['op_' + resources[i]] = $translate.instant(
					'settings.over_provision.exceeded',
					{ resource: $translate.instant('settings.' + resources[i]) });
			}
		}
		if(Object.keys(errors).length > 0) {
			message.error(errors);
			return false;
		}
		return true;
	}

	$scope.remove_limit = function(index) {
		if ($scope.resource_limits != undefined && typeof $scope.resource_limits[index] != 'undefined') {
			$scope.resource_limits.splice(index, 1);
		}
	}

	$scope.update_totals = function(name) {
		var op_field = name + '_overprovision';
		var total = name + '_total';
		var element_class = name + '_class';
		var element_total_class = name + '_total_class';
		if($scope.settings != undefined &&
			$scope.settings[op_field] != undefined &&
			$scope.settings[op_field].length > 0 &&
			!isNaN($scope.settings[op_field])
		) {
			$scope.system_resources[total] = $scope.system_resources[name] * $scope.settings[op_field] / 100;
		} else {
			$scope.system_resources[total] = $scope.system_resources[name];
		}
		var total_value = $scope.system_resources[total];
		if (name == 'memory') { total_value /= 1024*1024; }
		$scope[element_class] = $scope[element_total_class] = "";
		if($scope.allocated[name] > total_value) {
			$scope[element_class] = "error";
			$scope[element_total_class] = "number_error";
		}
	}

	$scope.add_row = function() {
		$scope.resource_limits.push({
			uuid:null,
			name:null,
			description:null,
			cpu:null,
			memory:null,
			diskspace:null
		});
	};

	$scope.whitelist = [];

	$scope.add_whitelist_entry = function() {
		$scope.whitelist.push({
			id:null,
			address:null,
			description:null,
			ssh:false,
			web:false,
		});
	};

	$scope.remove_whitelist_entry = function(index) {
		if ($scope.whitelist && $scope.whitelist[index]) {
			$scope.whitelist.splice(index, 1);
		}
	};

	ws.on('confirm-changes', function(event) {
		$app.confirm_token = event.token;
		$app.config_test_dialog = message.test_config();
		ws.reconnect();
	}, $scope);

	$scope.hide_footer = function() {
		$app.listing();
	};
	$scope.show_footer = function() {
		$app.creating($scope, 'footer.save');
	};
	$scope.show_footer();
	$app.tabs($route);

	$scope.external_interfaces = [];

	$q.all([
		ws.settings(),
		ws.interfaces(),
		ws.images(),
		ws.resource_limits(),
		ws.whitelist_services(),
		ws.host_whitelist(),
	]).then(([settings, interfaces, images, resource_plans, services, whitelist]) => {
		$scope.dump_uuid = $app.dump_uuid;
		if ($app.active_dump && $app.dump_uuid) { // if navigated away during dump
			$scope.dump_action = $translate.instant('settings.stop_n_download');
		} else {
			$scope.dump_action = $translate.instant('settings.start');
		}
		$scope.settings = settings.data.settings;
		$scope.system_resources = settings.data.system_resources;
		$scope.allocated = settings.data.allocated_resources;
		angular.forEach($scope.system_resources, function(value, name) {
			$scope.update_totals(name); //init totals
		});
		$scope.external_interfaces = interfaces.data;
		if($scope.settings.interface != undefined) {
			angular.forEach($scope.external_interfaces, function(interface) {
				if($scope.settings.interface == interface.name) {
					interface.checked = true;
				}
			})
		}
		$scope.images = Object.entries(images.data).map(([alias, image]) => {
			let name = alias;
			if (image.version != "") {
				name = `${name} (${image.version})`;
			}
			if (image.description != "") {
				name = `${name} - ${image.description}`;
			}
			return { name, alias };
		});
		if($scope.settings.pbx_image != undefined) {
			angular.forEach($scope.images, function(image) {
				if($scope.settings.pbx_image == image.alias) {
					image.checked = true;
				}
			})
		}
		angular.forEach(resource_plans.data, function(limit) {
			limit.memory = Math.round(limit.memory * 100) / 100;
		});
		$scope.resource_limits = resource_plans.data;
		$scope.services = services.data.filter(function(service) { return service.name != 'SIP' });
		$scope.whitelist = whitelist.data;
	}).catch($app.default_error_handler);
};
