return function ($app, ws, $q, $scope, $routeParams, $location, $mdDialog, $translate, toaster, $rootScope, multiselect, message, $route, $uibModal) {
	$scope.itemsPerPage = 25;
	$scope.currentPage = 1;
	$scope.whitelist = [];

	var add_to_scope = function(name, internal_port) {
		if($scope.services != undefined) {
			$scope.services.push({ // add to whitelist columns
				'name': name,
				'internal_port': internal_port,
				'forwarding': true,
			});
			angular.forEach($scope.whitelist, function(whitelist) {
				whitelist.enabled.push(false); //add column to existing rows
			});
		}
	}

	$scope.remove_did_route_row = function(index) {
		$uibModal.open({
			templateUrl: `/templates/remove_did_route_modal.html?${Date.now()}`,
		backdrop: 'static',
		backdropClass: 'modal-backdrop',
		animation: false,
		windowClass: 'show',
		scope: $scope,
		controller: function($scope, $uibModalInstance) {
			$scope.cancel = function() {
				$uibModalInstance.close();
			};
			$scope.ok = function() {
				var true_index = (($scope.currentPage - 1) * $scope.itemsPerPage) + index; // index start from 0
				if ($scope.did_routing && $scope.did_routing[true_index]) {
					$scope.did_routing.splice(true_index, 1);
				}
				toaster.success_centered($translate.instant('did_routing.route_removed'));
				$uibModalInstance.close();
			};
		},
		});
	}

	$scope.add_did_route_row = function(did_routing, uuid) {
		$uibModal.open({
			templateUrl: `/templates/add_did_route_modal.html?${Date.now()}`,
			backdrop: 'static',
			backdropClass: 'modal-backdrop',
			animation: false,
			windowClass: 'show',
			controller: function($scope, $uibModalInstance, ws) {
				$scope.cancel = function() {
					$uibModalInstance.close();
				};
				$scope.ok = function() {
					if($scope.pattern == null) {
						toaster.error($translate.instant('did_routing.pattern_empty'));
					} else if(!$scope.pattern_valid($scope.pattern)) {
						toaster.error($translate.instant('did_routing.pattern_invalid'));
					} else {
						did_routing.push({
							'uuid': uuid,
							'pattern': $scope.pattern,
						});
						$uibModalInstance.close();
						toaster.success_centered($translate.instant('did_routing.new_route_added'));
					}
				}
				$scope.pattern_valid = function(pattern) {
					if(pattern.length > 0) {
						var res = pattern.match(/^[0-9a-zA-Z?*,.+-_^\]\[]+$/g);
						if(res != null) {
							return true;
						}
					}
					return false;
				}
			},
		});
	};

	$scope.edit_did_route = function(data) {
		$uibModal.open({
			templateUrl: `/templates/edit_did_route_modal.html?${Date.now()}`,
			backdrop: 'static',
			backdropClass: 'modal-backdrop',
			animation: false,
			windowClass: 'show',
			controller: function($scope, $uibModalInstance, ws) {
				var vm = this;
				$scope.uuid = data.uuid;
				$scope.pattern = data.pattern;

				vm.editedItem = angular.copy(data);
				$scope.cancel = function() {
					$uibModalInstance.close();
				};
				$scope.ok = function() {
					if($scope.pattern == null) {
						toaster.error($translate.instant('did_routing.pattern_empty'));
					} else {
						vm.editedItem.uuid = $scope.uuid;
						vm.editedItem.pattern = $scope.pattern;
						angular.copy(vm.editedItem, data);
						$uibModalInstance.close();
						toaster.success_centered($translate.instant('did_routing.updated'));
					}
				}
			},
		});
	};

	$scope.add_service = function() {
		if(valid_service()) {
			add_to_scope($scope.new_service.name, $scope.new_service.internal_port);
			$scope.new_service = {}; // clean the new row
		}
	};

	var valid_service = function() {
		var errors = [];
		if(typeof $scope.new_service == "undefined" ||
			!('name' in $scope.new_service) ||
			!('internal_port' in $scope.new_service) ||
			$scope.new_service.name.length == 0 ||
			$scope.new_service.internal_port.length == 0
		) {
			errors['undefined'] = $translate.instant('tenant.service.undefined');
		}
		if(typeof $scope.new_service != "undefined" &&
			'internal_port' in $scope.new_service &&
			$scope.new_service.internal_port.length > 0 &&
			!isNumeric($scope.new_service.internal_port)
		){
			errors['internal_port'] = $translate.instant('tenant.service.internal_port.invalid');
		}
		if(Object.keys(errors).length > 0) {
			message.error(errors);
			return false;
		}
		return true;
	}

	var isNumeric = function(n) {
		return !isNaN(parseFloat(n)) && isFinite(n);
	}

	$scope.remove_service = function(index) {
		message.confirm({'content' : $translate.instant('tenant.service.remove.warning')}).then(function(result) {
			if(result == true) {
				if ($scope.services != undefined && typeof $scope.services[index] != 'undefined') {
					$scope.services.splice(index, 1);
				}
				angular.forEach($scope.whitelist, function(whitelist) {
					whitelist.enabled.splice(index, 1); //add column to existing rows
				});
			}
		}, function() {});
	};
	$scope.isActive = true;
	$scope.change_limit = function() {
		$scope.isActive = true;
		if ($rootScope.is_system_owner && $scope.tenant.resource_plan == "") {
			$scope.isActive = false;
		}
		$scope.resource_limits.withUUID($scope.tenant.resource_plan, function(limit) {
			$scope.tenant.memory = limit.memory;
			$scope.tenant.cpu = limit.cpu;
			$scope.tenant.diskspace = limit.diskspace;
		});
	}

	ws.on('container-app-usage-changed', function(event) {
		ws.container($scope.tenant.uuid).then((container) => {
			angular.forEach($scope.apps, (app) => {
				app.status = app.name in container.data.apps;
				app.usage = app.status ? container.data.apps[app.name] : "-";
			});
		});
	}, $scope);

	$scope.app_prompt = function(index) {
		const app = $scope.apps[index];
		if (app.status) {
			message.prompt_1({
				'title': app.displayname,
				'warning' : $translate.instant('tenant.active_paid_feature_warning'),
				'content' : app,
				'popup_text' : $translate.instant(`app.${app.name}.popup`),
			}).then(function() {
				app.status = false;
				if (app.toggle_status) {
					app.status = true;
					message.confirm_1({
						'title': 'global.note',
						'confirm': 'tenant.confirm',
						'cancel': 'tenant.cancel',
						'content' : 'tenant.full_upgrade_required'
					}).then(function(result) {
						if(!result) {
							app.status = false;
						}
					});
				}
			});
		} else {
			message.confirm_1({
				'title': 'global.warning',
				'theme': 'app_uninstall',
				'confirm': 'tenant.confirm',
				'cancel': 'tenant.cancel',
				'content' : $translate.instant('tenant.warning_remove_app', { name: app.displayname })
			}).then(function(result) {
				if(!result) { //remove app
					app.status = true;
				}
			});
		}
	};

	$scope.show_info = function(index) {
		message.info({'title': 'activate' , 'content' : $scope.apps[index]});
	};

	$scope.remove = function() {
		var confirm = $mdDialog.prompt()
			.theme('warning')
			.title($translate.instant('global.warning'))
			.htmlContent($translate.instant('tenant.remove_message') + ' <br/> ' + $translate.instant('tenant.confirm_message'))
			.cancel($translate.instant('tenant.cancel'))
			.ok($translate.instant('tenant.confirm'))
			.required(true);
		$mdDialog.show(confirm).then(function(result) {
			if($scope.tenant.name == result) {
				ws.remove_container($routeParams.uuid).then(function() {
					toaster.success($translate.instant('global.pending')); // no wok?
					$app.jump('/tenants');
				});
			} else {
				toaster.warning($translate.instant('tenant.name_no_match'));
			}
		}, function() {});
	};

	$scope.submit = function() {
		$scope.filter();
		if($scope.valid()) {
			if($scope.tenant.name != $scope.fromDB.name ||
				($scope.tenant.diskspace != $scope.fromDB.diskspace && $scope.tenant.status != 'stopped' )
			) {
				message.confirm({'content' : $translate.instant('tenant.restart_required.warning')}).then(function(result) {
					if(result == true) {
						do_update()
					}
				}, function() {});
			} else {
				do_update();
			}
		}
	};

	//filter in reverese to avoid array mutation
	$scope.filter = function() {
		if($scope.whitelist != undefined && $scope.whitelist.length > 0) {
			for(var i=$scope.whitelist.length - 1; i >= 0 ; i--) {
				if ($scope.whitelist[i].address == undefined || $scope.whitelist[i].address.length == 0) {
					$scope.whitelist.splice(i, 1);
				}
			}
		}
	}

	$scope.valid = function() {
		var errors = [];
		if($scope.tenant.name == undefined || $scope.tenant.name.length == 0) {
			errors['name'] = $translate.instant('tenant.name.invalid');
		}
		if($scope.tenant.resource_plan == undefined) {
			errors['resource_plan'] = $translate.instant('tenant.resource_plan.invalid');
		} else if($scope.tenant.resource_plan == "") { // custom
			if($scope.tenant.diskspace == undefined ||
				$scope.tenant.diskspace.length == 0 ||
				parseInt($scope.tenant.diskspace) == 0 ||
				parseInt($scope.tenant.diskspace) < 4
			) {
				errors['diskspace'] = $translate.instant('resource.diskspace.invalid');
			}
			if($scope.tenant.memory == undefined || $scope.tenant.memory < 0.5) {
				errors['memory'] = $translate.instant('resource.memory.invalid');
			}
			if($scope.tenant.cpu == undefined || isNaN($scope.tenant.cpu) || $scope.tenant.cpu.length == 0 || $scope.tenant.cpu < 0.1) {
				errors['cpu'] = $translate.instant('resource.cpu.invalid');
			} else {
				$scope.tenant.cpu = Math.round(parseFloat($scope.tenant.cpu) * 10) / 10;
				if($scope.tenant.cpu < 0.1) {
					errors['cpu'] = $translate.instant('resource.cpu.invalid');
				}
			}
		}
		if($scope.whitelist.length != 0) {
			if($scope.settings.interface == undefined || $scope.settings.interface.length == 0) {
				errors['interface'] = $translate.instant('settings.interface.invalid');
			}
			var used_addresses = [];
			angular.forEach($scope.whitelist, function(whitelist) {
				if (whitelist.address != undefined && whitelist.address.length > 0) {
					if(used_addresses.indexOf(whitelist.address) !== -1) {
						errors['duplicate_address'] = $translate.instant('settings.whitelist.ip.duplicate');
					}
					used_addresses.push(whitelist.address);
					if(whitelist.ami == false && whitelist.ssh == false &&
						whitelist.sip == false && whitelist.web == false
					) {
						errors['whitelist'] = $translate.instant('settings.whitelist.service.invalid');
					}
				}
			});
		}
		const basic_max = $scope.apps.find(app => app.name == "tmx-basic")?.max ?? -1;
		const advanced_max = $scope.apps.find(app => app.name == "tmx-advanced")?.max ?? -1;
		if (basic_max == 0 && advanced_max == 0) {
			errors['max'] = $translate.instant('tenant.zero_extensions');
		}
		if(Object.keys(errors).length > 0) {
			message.error(errors);
			return false;
		}
		return true;
	}

	var do_update = function() {
		if ($rootScope.is_system_owner) {
			$scope.tenant.owner = "";
			angular.forEach($scope.assigned_reseller, function(reseller) {
				if($scope.tenant != undefined) {
					$scope.tenant.owner = reseller.uuid;
				}
			});
		}
		ws.update_container($scope.tenant, $scope.whitelist, $scope.services, $scope.apps, $scope.did_routing).then(function() {
			$app.jump('/tenants');
			toaster.success($translate.instant('global.pending'));
		}).catch($app.default_error_handler);
	}

	$scope.cancel = function() {
		$app.jump('/tenants');
	};

	if ($scope.resellers == undefined) {
		$scope.resellers = [];
	}

	$scope.multiselect_l18n = multiselect.localization();

	$app.editing($scope);
	$app.tabs($route);

	$q.all([
		ws.container($routeParams.uuid),
		ws.resource_limits(),
		ws.apps(),
		ws.settings(),
		ws.container_did_routing($routeParams.uuid),
		ws.resellers(),
		ws.profile(),
	]).then(([container, resource_plans, apps, settings, routes, resellers, profile]) => {
		container.data.container.memory = Math.round(container.data.container.memory * 100) / 100;
		$scope.tenant = container.data.container;
		$scope.fromDB = angular.copy($scope.tenant);
		$scope.whitelist = container.data.whitelist;
		$scope.services = container.data.services;

		// needed to be grayed out for now. So hack it
		$scope.sip_index = null;
		angular.forEach($scope.services, function(service, index) {
			if(service.name == 'SIP') {
				$scope.sip_index = index;
			}
		});
		angular.forEach(resource_plans.data, function(limit) {
			limit.memory = Math.round(limit.memory * 100) / 100;
		});
		$scope.resource_limits = resource_plans.data;
		$scope.resource_limits.push({
			uuid: '',
			name: $translate.instant('tenant.custom'),
			description: $translate.instant('tenant.custom'),
			memory: 0.5,
			cpu: 0.8,
			diskspace: 4.0,
		});
		if($scope.tenant.resource_plan == "") {
			if ($rootScope.is_system_owner) {
				$scope.isActive = false;
			}
		}

		$scope.apps = [];
		for (const name in apps.data) {
			const active = container.data.container.apps[name]?.active ?? false;
			const max = container.data.container.apps[name]?.max ?? apps.data[name].max_default;
			const url = $translate.instant(`app.${name}.url`);
			$scope.apps.push({
				name: name,
				description: $translate.instant(`app.${name}.description`),
				displayname: $translate.instant(`app.${name}.displayname`),
				unit: $translate.instant(`app.${name}.unit`),
				status: active,
				usage: active ? container.data.container.apps[name]?.usage ?? 0 : "-",
				max: max >= 0 ? max : null,
				icon: $translate.instant(`app.${name}.icon`),
				url: (url != `app.${name}.url`) ? url : null,
				no_max: apps.data[name].no_max,
				force_enable: apps.data[name].force_enable,
				info: $translate.instant(`app.${name}.details`) != "",
			});
		}
		// extension apps are in fixed order, other apps are sorted by name
		$scope.apps = [
			$scope.apps.find(app => app.name == "extensions"),
			$scope.apps.find(app => app.name == "tmx-basic"),
			$scope.apps.find(app => app.name == "tmx-advanced"),
		].filter(app => !!app).concat(
			$scope.apps
				.filter(app => !["extensions", "tmx-basic", "tmx-advanced"].includes(app.name))
				.sort((a, b) => a.displayname.localeCompare(b.displayname))
		);
		$scope.settings = settings.data.settings;
		if ($rootScope.is_system_owner && !$scope.settings.username) {
			$scope.style = { display: 'block' };
		}
		$scope.did_routing = routes.data;
		$scope.resellers = resellers.data;
		$scope.resellers.withUUID($scope.tenant.owner, function(reseller) {
			reseller.checked = true;
		});
		$scope.profile = profile.data;
	}).catch($app.default_error_handler);
};
