return function ($app, ws, $q, $scope, $location, $mdDialog, $translate, toaster, $filter, message) {
	$app.listing();
	var port = $location.port();
	if (port != '') {
		port = ':' + port;
	}
	$scope.host = $location.host() + port;
	$scope.total_extensions = 0;
	$scope.total_tenants = 0;
	$scope.selected_tenants = [];

	$scope.go = function(destination) {
		$app.jump(destination);
	};
	var update_summary = function() {
		if($scope.tenants != undefined) {
			$scope.total_tenants = $scope.tenants.length;
			$scope.total_extensions = $scope.tenants.reduce(function(s, t) {
				if(t.extensions !== undefined) {
					return s + parseInt(t.extensions);
				}
				return s;
			}, 0);
		}
	};

	$scope.expandSelected = function(notification){
		angular.forEach($scope.notifications, function(item) {
			item.expanded=false;
		})
		notification.expanded=true;
	}

	$scope.cancel_bulk_action = function() {
		for (var i = $scope.notifications.length - 1; i >= 0; i--) {
			ws.cancel($scope.notifications[i].task_id).then(function() {
			}).catch($app.default_error_handler);
			if($scope.notifications[i].state != undefined &&
				!$scope.notifications[i].state.startsWith('starting') &&
				!$scope.notifications[i].state.startsWith('stopping') &&
				!$scope.notifications[i].state.startsWith('updating') &&
				!$scope.notifications[i].state.startsWith('upgrading')
			) {
				$scope.notifications.splice(i, 1);
			}
		}
		$scope.show_cancel_button = false; //hide cancel action button
	}

	$scope.bulk_action = function(action) {
		if($scope.selected_tenants) {
			message.password({
				'title': $translate.instant('tenant.title'),
				'theme': 'password',
				'confirm': 'tenant.confirm',
				'cancel': 'tenant.cancel',
				'content' : $translate.instant('tenant.confirm_action', {
					'action' : action,
					'tenants': $scope.selected_tenants.length
				})
			}).then(function(password) {
				if(password) {
					ws.validate_password(password).then(function() {
						$scope.show_cancel_button = true; //show cancel action button
						ws[action+'_containers']($scope.selected_tenants);
					}).catch($app.default_error_handler);
				}
			});
		}
	}

	$scope.isIndeterminate = function() {
		return ($scope.tenants && $scope.selected_tenants.length !== 0 &&
			$scope.selected_tenants.length !== $scope.tenants.length);
	};

	$scope.isChecked = function() {
		if($scope.selected_tenants && $scope.tenants) {
			return $scope.selected_tenants.length === $scope.tenants.length;
		}
		return false;
	};

	$scope.exists = function (item, list) {
		return list.indexOf(item) > -1;
	};

	$scope.toggle_all = function() {
		$scope.filteredArray = $filter('filter')($scope.tenants, $scope.searching);
		if($scope.filteredArray) {
			var rows = angular.element(document.querySelectorAll('table tbody tr'));
			if ($scope.selected_tenants.length == $scope.filteredArray.length) {
				$scope.is_checked = false;
				$scope.selected_tenants = [];
				rows.removeClass('active_row');
			} else {
				$scope.selected_tenants = [];
				angular.forEach($scope.filteredArray, function(tenant) {
					$scope.selected_tenants.push(tenant.uuid);
				});
				$scope.is_checked = true;
				rows.addClass('active_row');
			}
		}
	};

	$scope.toggle_selection = function(uuid) {
		var idx = $scope.selected_tenants.indexOf(uuid);
		if (idx > -1) {
			$scope.selected_tenants.splice(idx, 1);
		} else {
			$scope.selected_tenants.push(uuid);
		}
		angular.element(event.currentTarget).parent().parent().toggleClass("active_row");
	};

	// listener of query filter. On Change, update totals
	$scope.$watch('searching', function(newVal) {
		$scope.filteredArray = $filter('filter')($scope.tenants, newVal);
		if($scope.filteredArray) {
			$scope.total_tenants = $scope.filteredArray.length;
			$scope.total_extensions = $scope.filteredArray.reduce(function(s, t) {
				if(t.extensions !== undefined) {
					return s + parseInt(t.extensions);
				}
				return s;
			}, 0);
		}
	});

	ws.on('container-created', function(event) {
		event.data.apps = event.data.apps.map((name) => ({
			name: name,
			icon: $translate.instant(`app.${name}.icon`),
			displayname: $translate.instant(`app.${name}.displayname`),
		}));
		if ($scope.tenants != undefined) {
			$scope.tenants.push(event.data);
		} else {
			$scope.tenants = [event.data];
		}
		update_summary();
		toaster.success($translate.instant('tenant.created'));
	}, $scope);

	ws.on('container-started', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.status = 'running';
			});
			toaster.success(event.data.name, $translate.instant('tenant.started'));
		}
	}, $scope);

	ws.on('container-stopped', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.status = 'stopped';
			});
			toaster.success(event.data.name, $translate.instant('tenant.stopped'));
		}
	}, $scope);

	ws.on('container-removed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant, i, tenants) {
				tenants.splice(i, 1);
				update_summary();
				toaster.success($translate.instant('tenant.removed'));
			});
		}
	}, $scope);

	ws.on('container-renamed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.name = event.data.after;
			});
		}
	}, $scope);
	ws.on('container-system-name-changed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.system_name = event.data.after;
			});
		}
	}, $scope);

	ws.on('container-status-changed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.status = event.data.after;
				toaster.success(event.data.name, $translate.instant('tenant.'+tenant.status));
			});
		}
	}, $scope);

	ws.on('container-extensions-changed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.extensions = event.data.after;
				update_summary();
			});
		}
	}, $scope);

	ws.on('container-version-changed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.version = event.data.after;
			});
		}
	}, $scope);

	ws.on('container-serial-changed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.serial = event.data.after;
			});
		}
	}, $scope);

	ws.on('container-resource-plan-changed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				tenant.resource_plan = event.data.after;
			});
		}
	}, $scope);

	ws.on('container-owner-changed', function(event) {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, function(tenant) {
				if(event.data.after.length > 0) {
					tenant.displayname = event.data.after;
					tenant.owner = event.data.owner;
				} else {
					tenant.owner = null;
					tenant.username = "";
				}
			});
		}
	}, $scope);

	ws.on('container-app-installed', (event) => {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, (tenant) => {
				tenant.apps.push({
					name: event.data.app,
					icon: $translate.instant(`app.${event.data.app}.icon`),
					displayname: $translate.instant(`app.${event.data.app}.displayname`),
				});
				tenant.apps.sort((a, b) => { return a.name.localeCompare(b.name); });
			});
		}
	}, $scope);

	ws.on('container-app-removed', (event) => {
		if ($scope.tenants != undefined) {
			$scope.tenants.withUUID(event.data.uuid, (tenant) => {
				tenant.apps = tenant.apps.filter((app) => app.name != event.data.app);
			});
		}
	}, $scope);

	$q.all([
		ws.containers(),
		ws.resource_limits(),
		ws.resellers(),
	]).then(([containers, resource_plans, resellers]) => {
		const resource_plans_by_id = resource_plans.data.reduce((obj, resource_plan) => {
			obj[resource_plan.uuid] = resource_plan;
			return obj;
		}, {});
		const resellers_by_id = resellers.data.reduce((obj, reseller) => {
			obj[reseller.uuid] = reseller;
			return obj;
		}, {});
		for (container of containers.data) {
			const resource_plan = resource_plans_by_id[container.resource_plan];
			const reseller = resellers_by_id[container.owner];
			container.resource_plan = resource_plan?.name ?? $translate.instant("tenant.custom");
			container.reseller = reseller?.displayname ?? "";
			container.apps = Object.keys(container.apps).map((name) => {
				if (name != "extensions" && container.apps[name].active) {
					return {
						name: name,
						icon: $translate.instant(`app.${name}.icon`),
						displayname: $translate.instant(`app.${name}.displayname`),
					};
				}
			}).filter((app) => !!app);
			container.apps.sort((a, b) => { return a.name.localeCompare(b.name); });
		}
		$scope.tenants = containers.data;
		$scope.tenants.sort((a, b) => { return a.name.localeCompare(b.name); });
		update_summary();
	}).catch($app.default_error_handler);

	$scope.toggle_container = function(container_uuid, status, name) {
		if(status=='running') {
			var confirm = $mdDialog.prompt()
				.theme('prompt')
				.title($translate.instant('tenant.confirm_title'))
				.htmlContent($translate.instant('tenant.confirm_message'))
				.cancel($translate.instant('tenant.cancel'))
				.ok($translate.instant('tenant.confirm'))
				.required(true);
			$mdDialog.show(confirm).then(function(result) {
				if(name == result) {
					ws.stop_container(container_uuid).then(function() {
						toaster.success($translate.instant('global.pending'));
					});
				} else {
					toaster.warning($translate.instant('tenant.name_no_match'));
				}
			}, function() {});
		} else if(status=='stopped') {
			ws.start_container(container_uuid).then(function() {
				toaster.success($translate.instant('global.pending'));
			});
		}
	};
};
